#!c:/perl/bin/perl.exe
use warnings;
use strict;
use diagnostics;

# Read in the information from the two input files into
# separate hashes.

my $file_1_hash = read_file("Resources/result1.txt");
my $file_2_hash = read_file("Resources/result2.txt");


# As we don't have an array of the hash keys it's quicker
# to make up two anonymous arrays directly to pass to the
# compare_lists subroutine.  We could also have modified 
# the subroutine to accept hash references instead.

my @matches = compare_lists([keys(%$file_1_hash)],[keys(%$file_2_hash)]);

foreach my $match (@matches) {

	print join("\t",(
					$match,$file_1_hash->{$match}->{frequency},
					$file_2_hash->{$match}->{frequency},
					$file_1_hash->{$match}->{description}
					)),"\n";
	
}

sub read_file {
	
	# Reads in a data file and returns a hash reference 
	# containing all the useful data from that file.
	
	my ($file) = @_;
	
	my %data;
	
	open (my $in, $file) or die "Can't read $file: $!";
	while (<$in>) {
		chomp;
		
		# Skip headers
		next unless (/^Mm/);
		
		# Pick out the Unigene ID, frequency and description
		my ($ug,$freq,$desc) = (split(/\t/))[0,1,5];
		
		# Put something in if there isn't a description
		$desc = 'No description' unless ($desc);
		
		# Put the data into an anonymous hash and have this
		# as the value to the data hash indexed by the unigene id
		$data{$ug} = {
			frequency => $freq,
			description => $desc,
		};			
	}	
	
	# Return a reference to the data hash
	return \%data;
}


sub compare_lists {

	my ($list_a,$list_b) = @_;
	
	# The easist way to make this comparison
	# is to turn one list into a hash
	my %hash_a;
	
	$hash_a{$_} = 1 foreach (@$list_a);
	
	# Now we can iterate through the other list and see if
	# anything matches
	
	my @matches;
	
	foreach (@$list_b) {
		if (exists $hash_a{$_}) {
			push @matches, $_;	
		}	
	}
	
	return @matches;
}
